//////////////////////////////////////////////////////////////////////////////
// kindustrialbutton.cpp
// -------------------
// Industrial inspired window decoration for KDE
// -------------------
// Copyright (c) 2003 Gregg Edghill
// Please see the header file for copyright and license information.
//////////////////////////////////////////////////////////////////////////////

#include <kwin/options.h>

#include <qbitmap.h>
#include <qimage.h>
#include <qpainter.h>
#include <qpixmap.h>

#include "kindustrialbutton.h"
#include "kindustrialclient.h"
#include "misc.h"
#include "shadow.h"

using namespace KWinInternal;

// window button decorations
static unsigned char close_bits[] ={
   0xc3, 0xe7, 0x7e, 0x3c, 0x3c, 0x7e, 0xe7, 0xc3};
static unsigned char maximize_bits[] = {
   0xff, 0xff, 0x81, 0x81, 0x81, 0x81, 0x81, 0xff};
static unsigned char restore_bits[] = {
   0x00, 0x7e, 0x7e, 0x42, 0x42, 0x42, 0x7e, 0x00};
static unsigned char minimize_bits[] = {
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff};

static unsigned char empty_bits[] = {
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
static unsigned char help_bits[] = {
   0x7e, 0xe7, 0xc0, 0x70, 0x18, 0x00, 0x18, 0x18};
static unsigned char sticky_bits[] = {
   0x18, 0x7e, 0x66, 0xc3, 0xc3, 0x66, 0x7e, 0x18};
static unsigned char unsticky_bits[] = {
   0x18, 0x7e, 0x66, 0xdb, 0xdb, 0x66, 0x7e, 0x18};

static const int DECOSIZE   = 8;

KindustrialButton::KindustrialButton(KindustrialClient *parent, const char *name,
									 const QString& tip,
									 ButtonType type,
									 int size)
    : KWinButton(parent, name, tip), m_client(parent), m_lastMouse(0),
      m_size(size), m_type(type),
      hover(false),
      isSticky(false),
      isMaximized(false)
{
    setBackgroundMode(NoBackground);

    // the app icon should fit the button...
    if(m_size < 10) { m_size = 10; }

    setFixedSize(m_size, m_size);
}

KindustrialButton::~KindustrialButton()
{
}

QSize KindustrialButton::sizeHint() const
{
    return QSize(m_size, m_size);
}

void KindustrialButton::enterEvent(QEvent *e)
{
    hover = true;
    repaint(false);

    KWinButton::enterEvent(e);
}

void KindustrialButton::leaveEvent(QEvent *e)
{
    hover = false;
    repaint(false);

    KWinButton::leaveEvent(e);
}

void KindustrialButton::mousePressEvent(QMouseEvent* e)
{
    m_lastMouse = e->button();
    // pass on event after changing button to LeftButton
    QMouseEvent me(e->type(), e->pos(), e->globalPos(),
                   LeftButton, e->state());
    KWinButton::mousePressEvent(&me);
}

void KindustrialButton::mouseReleaseEvent(QMouseEvent* e)
{
    m_lastMouse = e->button();
    // pass on event after changing button to LeftButton
    QMouseEvent me(e->type(), e->pos(), e->globalPos(),
                   LeftButton, e->state());
    KWinButton::mouseReleaseEvent(&me);
}

void KindustrialButton::drawButton(QPainter *painter)
{
    if (!KindustrialHandler::initialized())
        return;

    bool active = m_client->isActive();
    QPixmap backgroundTile = m_client->getTitleBarTile(active);

    QColor titleBorder3 = KindustrialHandler::useTitleColor() ?
                          options->colorGroup(Options::TitleBar, active).background()
                          : colorGroup().background();
    QColor titleBorder1 = titleBorder3.dark(155); // dark

    QPixmap buffer;
    buffer.resize(width(), height());
    QPainter bufferPainter(&buffer);

    bufferPainter.drawTiledPixmap(0, 0, width(), width(), backgroundTile);
    if (hover)
    {
		if (isDown())	{bufferPainter.setPen(titleBorder3.dark(150));}
        else 			{bufferPainter.setPen(titleBorder3.dark(120));}
			
		bufferPainter.drawLine(2, 1, width()-3, 1);
        bufferPainter.drawLine(1, 2, 1, height()-3);
        bufferPainter.drawLine(2, height()-2, width()-3, height()-2);
        bufferPainter.drawLine(width()-2, 2, width()-2, height()-3);

		if(active)	{bufferPainter.fillRect(2,2,width()-4,height()-4,QColor(55,86,127));}
		else		{bufferPainter.fillRect(2,2,width()-4,height()-4,QColor(248,248,248));}
    }

    if (m_type == ButtonMenu)
    {
        QPixmap menuIcon = m_client->miniIcon();
        if (width() < menuIcon.width() || height() < menuIcon.height() ) {
            menuIcon.convertFromImage( menuIcon.convertToImage().smoothScale(width()-2, height()-2));
        }
        bufferPainter.drawPixmap((width()-menuIcon.width())/2, (height()-menuIcon.height())/2, menuIcon);
    }
    else
    {
        QBitmap deco;

        switch (m_type) {
        case ButtonClose:
            deco = QBitmap(DECOSIZE, DECOSIZE, close_bits, true);
            break;
        case ButtonMax:
            deco = QBitmap(DECOSIZE, DECOSIZE,
                           isMaximized ? restore_bits : maximize_bits,
                           true);
            break;
        case ButtonMin:
            deco = QBitmap(DECOSIZE, DECOSIZE, minimize_bits, true);
            break;
        case ButtonHelp:
            deco = QBitmap(DECOSIZE, DECOSIZE, help_bits, true);
            break;
        case ButtonSticky:
            deco = QBitmap(DECOSIZE, DECOSIZE,
                           isSticky ? unsticky_bits : sticky_bits,
                           true);
            break;
        default:
            deco = QBitmap(DECOSIZE, DECOSIZE, empty_bits, true);
        }
        deco.setMask(deco);

        int dX = (width()-DECOSIZE)/2;
        int dY = (height()-DECOSIZE)/2;

        if (isDown())
        {
            dY++;
            if (hover)
            {
                bufferPainter.setPen(titleBorder1.dark(120));
            }
            else
            {
                bufferPainter.setPen(titleBorder1);
            }
        }
        else
        {
            bufferPainter.setPen(titleBorder3.light(120) );
            bufferPainter.drawPixmap((width()-DECOSIZE)/2, (height()-DECOSIZE)/2+1, deco);
            if(hover)
            {
                bufferPainter.setPen(titleBorder1.light(120));
            }
            else
            {
                if(active)	{bufferPainter.setPen(QColor(123,165,218));}
				else		{bufferPainter.setPen(QColor(208,208,208));}

				bufferPainter.drawLine(2, 1, width()-3, 1);
            	bufferPainter.drawLine(1, 2, 1, height()-3);
            	bufferPainter.drawLine(2, height()-2, width()-3, height()-2);
           		bufferPainter.drawLine(width()-2, 2, width()-2, height()-3);

				if(active)	{bufferPainter.fillRect(2,2,width()-4,height()-4,QColor(55,86,127));}
				else		{bufferPainter.fillRect(2,2,width()-4,height()-4,QColor(248,248,248));}
            }
        }

		if(active)	{bufferPainter.setPen(QColor(255,255,255));}
		else		{bufferPainter.setPen(QColor(208,208,208));}
        bufferPainter.drawPixmap(dX, dY, deco);
    }

    bufferPainter.end();
    painter->drawPixmap(0, 0, buffer);
}
