//////////////////////////////////////////////////////////////////////////////
// kindustrialclient.cpp
// -------------------
// Industrial inspired window decoration for KDE
// -------------------
// Copyright (c) 2003 Gregg Edghill
// Please see the header file for copyright and license information.
//////////////////////////////////////////////////////////////////////////////

#include <klocale.h>
#include <kpixmap.h>
#include <kpixmapeffect.h>
#include <kwin/workspace.h>
#include <kwin/options.h>

#include <qbitmap.h>
#include <qdatetime.h>
#include <qfontmetrics.h>
#include <qimage.h>
#include <qlayout.h>
#include <qpainter.h>
#include <qpixmap.h>

#include "kindustrialclient.h"
#include "kindustrialclient.moc"
#include "kindustrialbutton.h"
#include "misc.h"
#include "shadow.h"

using namespace KWinInternal;

// global constants
static const int TOPMARGIN       = 2; // do not change
static const int DECOHEIGHT      = 1; // do not change
static const int SIDETITLEMARGIN = 3;

QColor penBlack = QColor(0,0,0);

// Default button layout
//  _____________________________________________
// |M|									 |H|I|A|X|
// |_____________________________________________|
//

const char default_left[]  = "M";
const char default_right[] = "HIAX";

KindustrialClient::KindustrialClient(Workspace *ws, WId w, QWidget *parent, const char *name)
    : Client (ws, w, parent, name, WResizeNoErase | WRepaintNoErase),
    mainLayout_(0),
    topSpacer_(0),
    titleSpacer_(0),
    leftTitleSpacer_(0), rightTitleSpacer_(0),
    decoSpacer_(0),
    leftSpacer_(0), rightSpacer_(0),
    bottomSpacer_(0),
    aCaptionBuffer(0), iCaptionBuffer(0),
    aTitleBarTile(0), iTitleBarTile(0),
    pixmaps_created(false),
    captionBufferDirty(true),
    closing(false),
    s_titleHeight(0),
    s_titleFont(QFont() )
{
    // store settings which maybe need to be changed "locally" or needed often
    s_titleHeight = isTool()?KindustrialHandler::titleHeightTool():KindustrialHandler::titleHeight();
    s_titleFont = isTool()?KindustrialHandler::titleFontTool():KindustrialHandler::titleFont();

    // for flicker-free redraws
    setBackgroundMode(NoBackground);

    _resetLayout();

    create_pixmaps();

    aCaptionBuffer = new QPixmap();
    iCaptionBuffer = new QPixmap();
    captionBufferDirty = true;
    repaint(titleSpacer_->geometry(), false);

}

KindustrialClient::~KindustrialClient()
{
    delete_pixmaps();

    delete aCaptionBuffer;
    delete iCaptionBuffer;

    for (int n=0; n<ButtonTypeCount; n++) {
        if (m_button[n]) delete m_button[n];
    }
}

void KindustrialClient::resizeEvent(QResizeEvent *e)
{
    Client::resizeEvent(e);
    doShape();
    repaint();
}

void KindustrialClient::paintEvent(QPaintEvent*)
{
    if (!KindustrialHandler::initialized()) return;

    if (captionBufferDirty)
        update_captionBuffer();

    bool active = isActive();

    QPainter painter(this);

    QRegion mask;

    QColor border3 = options->colorGroup(Options::TitleBlend, active).background();
    QColor titleBorder3 = KindustrialHandler::useTitleColor() ?
                          options->colorGroup(Options::TitleBar, active).background()
                          : colorGroup().background();

	QColor titleBorder4 = hsvRelative(titleBorder3, 0, 0, 70); // light

    QRect topRect(topSpacer_->geometry());
    QRect titleRect(titleSpacer_->geometry());
    QRect leftTitleRect(leftTitleSpacer_->geometry());
    QRect rightTitleRect(rightTitleSpacer_->geometry());
    QRect decoRect(decoSpacer_->geometry());
    QRect leftRect(leftSpacer_->geometry());
    QRect rightRect(rightSpacer_->geometry());
    QRect bottomRect(bottomSpacer_->geometry());
    QRect tempRect;

    // topSpacer
    if(topRect.height() > 0)
    {
		painter.setPen(penBlack);

        painter.drawLine(topRect.left()+4, topRect.top(),
                         topRect.right()-4, topRect.top() );
        painter.drawLine(topRect.left()+2, topRect.top()+1,
                         topRect.left()+2+2, topRect.top()+1 );
        painter.drawLine(topRect.right()-2-2, topRect.top()+1,
                         topRect.right()-2, topRect.top()+1 );
        painter.setPen(titleBorder4 );
        painter.drawLine(topRect.left()+4, topRect.top()+1,
                         topRect.right()-4, topRect.top()+1 );
    }

    // leftTitleSpacer
    if(leftTitleRect.width() > 0)
    {
		painter.setPen(penBlack);

        painter.drawLine(leftTitleRect.left(), leftTitleRect.top()+2,
                         leftTitleRect.left(), leftTitleRect.bottom() );
        painter.drawLine(leftTitleRect.left()+1, leftTitleRect.top(),
                         leftTitleRect.left()+1, leftTitleRect.top()+2 );

        mask  = QRegion(leftTitleRect);
        mask -= QRegion(leftTitleRect.left(), leftTitleRect.top(), 1, leftTitleRect.height() );
        mask -= QRegion(leftTitleRect.left()+1, leftTitleRect.top(), 1, 2 );
        painter.setClipRegion(mask);
        painter.setClipping(true);
        painter.drawTiledPixmap(leftTitleRect, active ? *aTitleBarTile : *iTitleBarTile );
        painter.setClipping(false);
    }

    // Space under the left button group
    painter.drawTiledPixmap(leftTitleRect.right()+1, titleRect.top(),
                            (titleRect.left()-1)-leftTitleRect.right(), titleRect.height(),
                            active ? *aTitleBarTile : *iTitleBarTile );


    // rightTitleSpacer
    if(rightTitleRect.width() > 0)
    {
		painter.setPen(penBlack);

        painter.drawLine(rightTitleRect.right(), rightTitleRect.top()+2,
                         rightTitleRect.right(), rightTitleRect.bottom() );
        painter.drawLine(rightTitleRect.right()-1, rightTitleRect.top(),
                         rightTitleRect.right()-1, rightTitleRect.top()+2 );

        mask  = QRegion(rightTitleRect);
        mask -= QRegion(rightTitleRect.right(), rightTitleRect.top(), 1, leftTitleRect.height() );
        mask -= QRegion(rightTitleRect.right()-1, rightTitleRect.top(), 1, 2 );
        painter.setClipRegion(mask);
        painter.setClipping(true);
        painter.drawTiledPixmap(rightTitleRect, active ? *aTitleBarTile : *iTitleBarTile );
        painter.setClipping(false);
    }

    // Space under the right button group
    painter.drawTiledPixmap(titleRect.right()+1, titleRect.top(),
                            (rightTitleRect.left()-1)-titleRect.right(), titleRect.height(),
                            active ? *aTitleBarTile : *iTitleBarTile );

    // titleSpacer
    QPixmap *titleBfrPtr = active ? aCaptionBuffer : iCaptionBuffer;
    if(titleRect.width() > 0 && titleBfrPtr != 0)
    {
        const int titleMargin = 5; // 5 px between title and buttons

        int tX, tW;
        switch (KindustrialHandler::titleAlign())
        {
        // AlignCenter
        case Qt::AlignHCenter:
            tX = (titleBfrPtr->width() > titleRect.width()-2*titleMargin) ?
                    (titleRect.left()+titleMargin)
                    : titleRect.left()+(titleRect.width()- titleBfrPtr->width() )/2;
            tW = (titleBfrPtr->width() > titleRect.width()-2*titleMargin) ?
                    (titleRect.width()-2*titleMargin)
                    : titleBfrPtr->width();
            break;
        // AlignRight
        case Qt::AlignRight:
            tX = (titleBfrPtr->width() > titleRect.width()-2*titleMargin) ?
                    (titleRect.left()+titleMargin)
                    : titleRect.right()-titleMargin-titleBfrPtr->width();
            tW = (titleBfrPtr->width() > titleRect.width()-2*titleMargin) ?
                    (titleRect.width()-2*titleMargin)
                    : titleBfrPtr->width();
            break;
        // AlignLeft
        default:
            tX = (titleRect.left()+titleMargin);
            tW = (titleBfrPtr->width() > titleRect.width()-2*titleMargin) ?
                    (titleRect.width()-2*titleMargin)
                    : titleBfrPtr->width();
        }

        if(tW > 0)
        {
            painter.drawTiledPixmap(tX, titleRect.top(),
                                    tW, titleRect.height(),
                                    *titleBfrPtr);
        }

        painter.drawTiledPixmap(titleRect.left(), titleRect.top(),
                                tX-titleRect.left(), titleRect.height(),
                                active ? *aTitleBarTile : *iTitleBarTile);

        painter.drawTiledPixmap(tX+tW, titleRect.top(),
                                titleRect.right()-(tX+tW)+1, titleRect.height(),
                                active ? *aTitleBarTile : *iTitleBarTile);
    }
    titleBfrPtr = 0;

	// basic layout:
    //  _______________________________________________________________
    // |                         topSpacer                             |
    // |_______________________________________________________________|
    // | leftTitleSpacer | btns | titleSpacer | bts | rightTitleSpacer |
    // |_________________|______|_____________|_____|__________________|
    // |                         decoSpacer                            |
    // |_______________________________________________________________|
    // | |                                                           | |
    // | |                      windowWrapper                        | |
    // | |                                                           | |
    // |leftSpacer                                          rightSpacer|
    // |_|___________________________________________________________|_|
    // |                           bottomSpacer                        |
    // |_______________________________________________________________|
    //

    // decoSpacer
    if(decoRect.height() > 0)
    {
		// |_ __	left
		// | |
		painter.setPen(titleBorder3);
		//painter.drawLine(decoRect.left()+1, decoRect.top(), decoRect.left()+3, decoRect.top() );
  		painter.drawLine(leftRect.left()+1,decoRect.top(),rightRect.left()+3,decoRect.top() );

		//  __ _|	right
		//    | |
		//painter.drawLine(decoRect.right()-3, decoRect.top(), decoRect.right()-1, decoRect.top() );
		painter.drawLine(rightRect.right()-3,decoRect.top(),rightRect.right()-1,decoRect.top() );

		painter.setPen(penBlack);
		painter.drawPoint(decoRect.left(), decoRect.top() );
		painter.drawPoint(decoRect.right(), decoRect.top() );

		if(!active)	{painter.setPen(QColor(208,208,208));}
        //painter.drawLine(decoRect.left()+4, decoRect.top(), decoRect.right()-4, decoRect.top() );
		painter.drawLine(leftRect.right()+1,decoRect.top(),rightRect.left()-1,decoRect.top());
    }

    if(leftRect.height() == 1 && rightRect.height() == 1) // the window is shaded... hopefully
    {
        // let's orient on the bottomSpacer which should have the prper x coords...
        painter.setPen(penBlack);
        painter.drawPoint(bottomRect.left(), leftRect.top() );
        painter.drawPoint(bottomRect.right(), rightRect.top() );
        painter.setPen(border3 );
        painter.drawLine(bottomRect.left()+4, leftRect.top(), bottomRect.right()-4, rightRect.top() );

    }
    else
    {
        // leftSpacer
        if(leftRect.width() > 0 && leftRect.height() > 0)
        {
			// outer-border
			painter.setPen(penBlack);
            painter.drawLine(leftRect.left(), leftRect.top(),
                             leftRect.left(), leftRect.bottom() );
			//inner-border
			if(!active)	{painter.setPen(QColor(208,208,208));}
            painter.drawLine(leftRect.right(), leftRect.top(),
                             leftRect.right(), leftRect.bottom() );

            tempRect.setCoords(leftRect.left()+1, leftRect.top(),
                               leftRect.right()-1, leftRect.bottom() );
            painter.fillRect(tempRect, titleBorder3);
        }

        // rightSpacer
        if(rightRect.width() > 0 && rightRect.height() > 0)
        {
			// outer -border
			painter.setPen(penBlack);
            painter.drawLine(rightRect.right(), rightRect.top(),
                             rightRect.right(), rightRect.bottom() );
			// inner-border
			if(!active)	{painter.setPen(QColor(208,208,208));}
            painter.drawLine(rightRect.left(), rightRect.top(),
                             rightRect.left(), rightRect.bottom() );

            tempRect.setCoords(rightRect.left()+1, rightRect.top(),
                               rightRect.right()-1, rightRect.bottom() );
            painter.fillRect(tempRect, titleBorder3);
        }
    }

    // bottomSpacer
    if(bottomRect.height() > 0)
    {

		painter.setPen(penBlack);
        painter.drawLine(bottomRect.left(), bottomRect.top(),
                         bottomRect.left(), bottomRect.bottom()-1 );
        painter.drawLine(bottomRect.right(), bottomRect.top(),
                         bottomRect.right(), bottomRect.bottom()-1 );

        painter.setPen(titleBorder3);
        painter.drawLine(bottomRect.left()+1, bottomRect.bottom()-1,
                         bottomRect.right()-1, bottomRect.bottom()-1 );

        tempRect.setCoords(bottomRect.left()+1, bottomRect.top(),
                           bottomRect.right()-1, bottomRect.bottom()-2);
        painter.fillRect(tempRect, titleBorder3);

		painter.setPen(penBlack);
		// bottom-left corner
		painter.drawLine(bottomRect.left()+1, bottomRect.top(),
						 bottomRect.left()+1, bottomRect.bottom()-2);
		painter.drawLine(bottomRect.left()+2,bottomRect.bottom()-1,
						 bottomRect.left()+3,bottomRect.bottom()-1);
		// bottom-right corner
		painter.drawLine(bottomRect.right()-1, bottomRect.top(),
						 bottomRect.right()-1, bottomRect.bottom()-2);
		painter.drawLine(bottomRect.right()-3,bottomRect.bottom()-1,
						 bottomRect.right()-2,bottomRect.bottom()-1);
		// outer-border
		painter.drawLine(bottomRect.left()+4, bottomRect.bottom(),
                         bottomRect.right()-4, bottomRect.bottom() );
		// inner-border
		if(!active)	{painter.setPen(QColor(208,208,208));}
		painter.drawLine(leftRect.right()+1, bottomRect.top(),
                         rightRect.left()-1, bottomRect.top() );
    }
}

void KindustrialClient::showEvent(QShowEvent *)
{
    doShape();
    repaint();
}


void KindustrialClient::windowWrapperShowEvent(QShowEvent *)
{
    doShape();
    repaint();
}

void KindustrialClient::mouseDoubleClickEvent(QMouseEvent *e)
{
    if (titleSpacer_->geometry().contains(e->pos()))
        workspace()->performWindowOperation(this,
            options->operationTitlebarDblClick());
}

void KindustrialClient::doShape()
{
    QRegion mask(0, 0, width(), height());

    int r(width());
    int b(height());

    if(topSpacer_->geometry().height() > 0)
    {
        // Remove top-left corner.
        if(leftTitleSpacer_->geometry().width() > 0)
        {
            mask -= QRegion(0, 0, 1, 4);
            mask -= QRegion(1, 0, 3, 1);
            mask -= QRegion(1, 1, 1, 1);
        }
        // Remove top-right corner.
        if(rightTitleSpacer_->geometry().width() > 0)
        {
            mask -= QRegion(r-1, 0, 1, 4);
            mask -= QRegion(r-4, 0, 3, 1);
            mask -= QRegion(r-2, 1, 1, 1);
        }
    }

    // Remove bottom-left corner and bottom-right corner.
    if(bottomSpacer_->geometry().height() > 0)
    {
		//bottom-left
		//mask -= QRegion(0, b-1, 1, 1);
		mask -= QRegion(1,b-2,1,1);
		mask -= QRegion(0,b-4,1,4);
		mask -= QRegion(0,b-1,4,1);
		//bottom-right
		//mask -= QRegion(r-1, b-1, 1, 1);
		mask -= QRegion(r-2,b-2,1,1);
		mask -= QRegion(r-1,b-4,1,4);
		mask -= QRegion(r-4,b-1,4,1);
    }

    setMask(mask);
}

void KindustrialClient::_resetLayout()
{
    // basic layout:
    //  _______________________________________________________________
    // |                         topSpacer                             |
    // |_______________________________________________________________|
    // | leftTitleSpacer | btns | titleSpacer | bts | rightTitleSpacer |
    // |_________________|______|_____________|_____|__________________|
    // |                         decoSpacer                            |
    // |_______________________________________________________________|
    // | |                                                           | |
    // | |                      windowWrapper                        | |
    // | |                                                           | |
    // |leftSpacer                                          rightSpacer|
    // |_|___________________________________________________________|_|
    // |                           bottomSpacer                        |
    // |_______________________________________________________________|
    //

    if (!KindustrialHandler::initialized()) return;

    delete mainLayout_;

    delete topSpacer_;
    delete titleSpacer_;
    delete leftTitleSpacer_;
    delete rightTitleSpacer_;
    delete decoSpacer_;
    delete leftSpacer_;
    delete rightSpacer_;
    delete bottomSpacer_;

    mainLayout_ = new QVBoxLayout(this, 0, 0);

    topSpacer_        = new QSpacerItem(1, TOPMARGIN, QSizePolicy::Expanding, QSizePolicy::Fixed);
    titleSpacer_      = new QSpacerItem(1, s_titleHeight,
                                        QSizePolicy::Expanding, QSizePolicy::Fixed);
    leftTitleSpacer_  = new QSpacerItem(SIDETITLEMARGIN, s_titleHeight,
                                        QSizePolicy::Fixed, QSizePolicy::Fixed);
    rightTitleSpacer_ = new QSpacerItem(SIDETITLEMARGIN, s_titleHeight,
                                        QSizePolicy::Fixed, QSizePolicy::Fixed);
    decoSpacer_       = new QSpacerItem(1, DECOHEIGHT, QSizePolicy::Expanding, QSizePolicy::Fixed);
    leftSpacer_       = new QSpacerItem(KindustrialHandler::borderSize(), 1,
                                        QSizePolicy::Fixed, QSizePolicy::Expanding);
    rightSpacer_      = new QSpacerItem(KindustrialHandler::borderSize(), 1,
                                        QSizePolicy::Fixed, QSizePolicy::Expanding);
    bottomSpacer_     = new QSpacerItem(1, KindustrialHandler::borderSize(),
                                        QSizePolicy::Expanding, QSizePolicy::Fixed);

    // top
    mainLayout_->addItem(topSpacer_);

    // title
    QHBoxLayout *titleLayout_ = new QHBoxLayout(mainLayout_, 0, 0);

    // sizeof(...) is calculated at compile time
    memset(m_button, 0, sizeof(KindustrialButton *) * ButtonTypeCount);

    titleLayout_->addItem(leftTitleSpacer_);
    addButtons(titleLayout_,
               options->customButtonPositions() ? options->titleButtonsLeft() : QString(default_left),
               s_titleHeight-1);
    titleLayout_->addItem(titleSpacer_);
    addButtons(titleLayout_,
               options->customButtonPositions() ? options->titleButtonsRight() : QString(default_right),
               s_titleHeight-1);
    titleLayout_->addItem(rightTitleSpacer_);

    // deco
    mainLayout_->addItem(decoSpacer_);

    //Mid
    QHBoxLayout * midLayout   = new QHBoxLayout(mainLayout_, 0, 0);
    midLayout->addItem(leftSpacer_);
    midLayout->addWidget(windowWrapper());
    midLayout->addItem(rightSpacer_);

    //Bottom
    mainLayout_->addItem(bottomSpacer_);


}

void KindustrialClient::addButtons(QBoxLayout *layout, const QString& s, int buttonSize)
{
    if (s.length() > 0) {
        for (unsigned n=0; n < s.length(); n++) {
            switch (s[n]) {
              case 'M': // Menu button
                  if (!m_button[ButtonMenu]){
                      m_button[ButtonMenu] = new KindustrialButton(this, "menu", i18n("Menu"), ButtonMenu, buttonSize);
                      connect(m_button[ButtonMenu], SIGNAL(pressed()), this, SLOT(menuButtonPressed()));
                      connect(m_button[ButtonMenu], SIGNAL(released()), this, SLOT(menuButtonReleased()));
                      layout->addWidget(m_button[ButtonMenu], 0, Qt::AlignHCenter | Qt::AlignTop);
                  }
                  break;
              case 'S': // Sticky button
                  if (!m_button[ButtonSticky]){
                      m_button[ButtonSticky] = new KindustrialButton(this, "sticky", i18n("Sticky"), ButtonSticky, buttonSize);
                      connect(m_button[ButtonSticky], SIGNAL(clicked()), this, SLOT(toggleSticky()));
                      layout->addWidget(m_button[ButtonSticky], 0, Qt::AlignHCenter | Qt::AlignTop);
                  }
                  break;
              case 'H': // Help button
                  if ((!m_button[ButtonHelp]) && providesContextHelp()){
                      m_button[ButtonHelp] = new KindustrialButton(this, "help", i18n("Help"), ButtonHelp, buttonSize);
                      connect(m_button[ButtonHelp], SIGNAL(clicked()), this, SLOT(contextHelp()));
                      layout->addWidget(m_button[ButtonHelp], 0, Qt::AlignHCenter | Qt::AlignTop);
                  }
                  break;
              case 'I': // Minimize button
                  if ((!m_button[ButtonMin]) && isMinimizable()){
                      m_button[ButtonMin] = new KindustrialButton(this, "iconify", i18n("Minimize"), ButtonMin, buttonSize);
                      connect(m_button[ButtonMin], SIGNAL(clicked()), this, SLOT(iconify()));
                      layout->addWidget(m_button[ButtonMin], 0, Qt::AlignHCenter | Qt::AlignTop);
                  }
                  break;
              case 'A': // Maximize button
                  if ((!m_button[ButtonMax]) && isMaximizable()){
                      m_button[ButtonMax] = new KindustrialButton(this, "maximize", i18n("Maximize"), ButtonMax, buttonSize);
                      connect(m_button[ButtonMax], SIGNAL(clicked()), this, SLOT(maxButtonPressed()));
                      layout->addWidget(m_button[ButtonMax], 0, Qt::AlignHCenter | Qt::AlignTop);
                  }
                  break;
              case 'X': // Close button
                  if ((!m_button[ButtonClose]) && isCloseable()){
                      m_button[ButtonClose] = new KindustrialButton(this, "close", i18n("Close"), ButtonClose, buttonSize);
                      connect(m_button[ButtonClose], SIGNAL(clicked()), this, SLOT(closeWindow()));
                      layout->addWidget(m_button[ButtonClose], 0, Qt::AlignHCenter | Qt::AlignTop);
                  }
                  break;
              case '_': // Spacer item
                  layout->addSpacing(3); // add a 3 px spacing...
            }
	}
    }
}

void KindustrialClient::captionChange(const QString &)
{
    captionBufferDirty = true;
    repaint(titleSpacer_->geometry(), false);
}

Client::MousePosition KindustrialClient::mousePosition(const QPoint &point) const
{
    const int corner = 24;
    MousePosition pos = Center;

    // often needed coords..
    QRect topRect(topSpacer_->geometry());
    QRect decoRect(decoSpacer_->geometry());
    QRect leftRect(leftSpacer_->geometry());
    QRect leftTitleRect(leftTitleSpacer_->geometry());
    QRect rightRect(rightSpacer_->geometry());
    QRect rightTitleRect(rightTitleSpacer_->geometry());
    QRect bottomRect(bottomSpacer_->geometry());

    if(bottomRect.contains(point)) {
        if      (point.x() <= bottomRect.left()+corner)  pos = BottomLeft;
        else if (point.x() >= bottomRect.right()-corner) pos = BottomRight;
        else                                             pos = Bottom;
    }
    else if(leftRect.contains(point)) {
        if      (point.y() <= topRect.top()+corner)       pos = TopLeft;
        else if (point.y() >= bottomRect.bottom()-corner) pos = BottomLeft;
        else                                              pos = Left;
    }
    else if(leftTitleRect.contains(point)) {
        if      (point.y() <= topRect.top()+corner)       pos = TopLeft;
        else                                              pos = Left;
    }
    else if(rightRect.contains(point)) {
        if      (point.y() <= topRect.top()+corner)       pos = TopRight;
        else if (point.y() >= bottomRect.bottom()-corner) pos = BottomRight;
        else                                              pos = Right;
    }
    else if(rightTitleRect.contains(point)) {
        if      (point.y() <= topRect.top()+corner)       pos = TopRight;
        else                                              pos = Right;
    }
    else if(topRect.contains(point)) {
        if      (point.x() <= topRect.left()+corner)      pos = TopLeft;
        else if (point.x() >= topRect.right()-corner)     pos = TopRight;
        else                                              pos = Top;
    }
    else if(decoRect.contains(point)) {
        if(point.x() <= leftTitleRect.right()) {
            if(point.y() <= topRect.top()+corner)         pos = TopLeft;
            else                                          pos = Left;
        }
        else if(point.x() >= rightTitleRect.left()) {
            if(point.y() <= topRect.top()+corner)         pos = TopRight;
            else                                          pos = Right;
        }
    }

    return pos;
}

void KindustrialClient::iconChange()
{
    if (m_button[ButtonMenu])
    {
        m_button[ButtonMenu]->repaint(false);
    }
}

void KindustrialClient::activeChange(bool)
{
    for (int n=0; n<ButtonTypeCount; n++)
        if (m_button[n]) m_button[n]->repaint(false);
    repaint(false);
}

void KindustrialClient::maximizeChange(bool m)
{
    if (!KindustrialHandler::initialized()) return;

	if( m && (maximizeMode() == MaximizeFull) && KindustrialHandler::shrinkBorders() )
	{
    	topSpacer_->changeSize(1, 0, QSizePolicy::Expanding, QSizePolicy::Fixed);
		bottomSpacer_->changeSize(1, 3, QSizePolicy::Expanding, QSizePolicy::Fixed);
        if (! options->moveResizeMaximizedWindows)
		{
                // normal size! KWin then cuts these borders on its own
    		leftSpacer_->changeSize(KindustrialHandler::borderSize(), 1,
								QSizePolicy::Fixed, QSizePolicy::Expanding);
    		rightSpacer_->changeSize(KindustrialHandler::borderSize(), 1,
								QSizePolicy::Fixed, QSizePolicy::Expanding);

    		leftTitleSpacer_->changeSize(KindustrialHandler::borderSize(), s_titleHeight,
                            	QSizePolicy::Fixed, QSizePolicy::Fixed);
    		rightTitleSpacer_->changeSize(KindustrialHandler::borderSize(), s_titleHeight,
                            	QSizePolicy::Fixed, QSizePolicy::Fixed);
        }
		else
		{
    		leftSpacer_->changeSize(0, 1, QSizePolicy::Fixed, QSizePolicy::Expanding);
    		rightSpacer_->changeSize(0, 1, QSizePolicy::Fixed, QSizePolicy::Expanding);

    		leftTitleSpacer_->changeSize(0, s_titleHeight,
                            	QSizePolicy::Fixed, QSizePolicy::Fixed);
    		rightTitleSpacer_->changeSize(0, s_titleHeight,
                            	QSizePolicy::Fixed, QSizePolicy::Fixed);
    	}
	}
    else
    {
    	topSpacer_->changeSize(1, TOPMARGIN, QSizePolicy::Expanding, QSizePolicy::Fixed);
    	bottomSpacer_->changeSize(1, KindustrialHandler::borderSize(),
    							QSizePolicy::Expanding, QSizePolicy::Fixed);
    	leftSpacer_->changeSize(KindustrialHandler::borderSize(), 1,
                        		QSizePolicy::Fixed, QSizePolicy::Expanding);
    	rightSpacer_->changeSize(KindustrialHandler::borderSize(), 1,
                        		QSizePolicy::Fixed, QSizePolicy::Expanding);
    	leftTitleSpacer_->changeSize(SIDETITLEMARGIN, s_titleHeight,
                        		QSizePolicy::Fixed, QSizePolicy::Fixed);
    	rightTitleSpacer_->changeSize(SIDETITLEMARGIN, s_titleHeight,
                        		QSizePolicy::Fixed, QSizePolicy::Fixed);
	}
    layout()->activate();
    repaint( false );

    if (m_button[ButtonMax])
    {
        m_button[ButtonMax]->setMaximized(m);
        m_button[ButtonMax]->setTipText(m ? i18n("Restore") : i18n("Maximize"));
    }

    doShape();
}

void KindustrialClient::stickyChange(bool s)
{
    if (m_button[ButtonSticky])
    {
        m_button[ButtonSticky]->setSticky(s);
        m_button[ButtonSticky]->setTipText(s ? i18n("Un-Sticky") : i18n("Sticky"));
    }
}

void KindustrialClient::maxButtonPressed()
{
    if (m_button[ButtonMax])
    {
        switch (m_button[ButtonMax]->lastMousePress())
        {
          case MidButton:   maximize(MaximizeVertical);   break;
          case RightButton: maximize(MaximizeHorizontal); break;
          default:          maximize();
        }
    }
}

void KindustrialClient::menuButtonPressed()
{
    static QTime* t = NULL;
    static KindustrialClient* lastClient = NULL;
    if (t == NULL)
        t = new QTime;
    bool dbl = (lastClient == this && t->elapsed() <= QApplication::doubleClickInterval());
    lastClient = this;
    t->start();
    if( !dbl )
    {
        QPoint pos = m_button[ButtonMenu]->mapToGlobal(
            m_button[ButtonMenu]->rect().bottomLeft() );
        workspace()->showWindowMenu( pos.x(), pos.y(), this );
        m_button[ButtonMenu]->setDown(false);
    }
    else
       closing = true;
}

void KindustrialClient::menuButtonReleased()
{
    if(closing)
        closeWindow();
}

void KindustrialClient::create_pixmaps()
{
    if(pixmaps_created)
        return;

    QColor titleBorder3 = KindustrialHandler::useTitleColor() ?
                          options->colorGroup(Options::TitleBar, true).background()
                          : colorGroup().background();
    KPixmap tempPixmap;
    tempPixmap.resize(1, s_titleHeight );
    KPixmapEffect::gradient(tempPixmap,
                            hsvRelative(titleBorder3, 0, 0, 30),
                            hsvRelative(titleBorder3, 0, 0, -10),
                            KPixmapEffect::VerticalGradient);
    aTitleBarTile = new QPixmap(1, s_titleHeight );
    QPainter painter;
    painter.begin(aTitleBarTile);
    painter.drawPixmap(0, 0, tempPixmap);
    painter.end();

    titleBorder3 = KindustrialHandler::useTitleColor() ?
                   options->colorGroup(Options::TitleBar, false).background()
                   : colorGroup().background();
    tempPixmap.resize(1, s_titleHeight );
    KPixmapEffect::gradient(tempPixmap,
                            hsvRelative(titleBorder3, 0, 0, 5),
                            hsvRelative(titleBorder3, 0, 0, -10),
                            KPixmapEffect::VerticalGradient);
    iTitleBarTile = new QPixmap(1, s_titleHeight );
    painter.begin(iTitleBarTile);
    painter.drawPixmap(0, 0, tempPixmap);
    painter.end();

    pixmaps_created = true;
}

void KindustrialClient::delete_pixmaps()
{
    delete aTitleBarTile;
    aTitleBarTile = 0;

    delete iTitleBarTile;
    iTitleBarTile = 0;

    pixmaps_created = false;
}

void KindustrialClient::update_captionBuffer()
{
    if (!KindustrialHandler::initialized()) return;

    const uint maxCaptionLength = 300; // truncate captions longer than this!
    QString caption(caption() );
    if (caption.length() > maxCaptionLength) {
        caption.truncate(maxCaptionLength);
        caption.append(" [...]");
    }

    QFontMetrics fm(s_titleFont);
    int captionWidth  = fm.width(caption);

    QPixmap textPixmap;
    QPainter painter;
    if(KindustrialHandler::titleShadow())
    {
        // prepare the shadow
        textPixmap = QPixmap(captionWidth+2*2, s_titleHeight ); // 2*2 px shadow space
        textPixmap.fill(penBlack);
        textPixmap.setMask( textPixmap.createHeuristicMask(TRUE) );
        painter.begin(&textPixmap);
        painter.setFont(s_titleFont);
        painter.setPen(white);
        painter.drawText(textPixmap.rect(), AlignCenter, caption );
        painter.end();
    }

    QImage shadow;
    ShadowEngine se;

    // active
    aCaptionBuffer->resize(captionWidth+4, s_titleHeight ); // 4 px shadow
    painter.begin(aCaptionBuffer);
    painter.drawTiledPixmap(aCaptionBuffer->rect(), *aTitleBarTile);
    if(KindustrialHandler::titleShadow())
    {
        shadow = se.makeShadow(textPixmap, QColor(0, 0, 0));
        painter.drawImage(1, 1, shadow);
    }
    painter.setFont(s_titleFont);
    painter.setPen(options->color(Options::Font, true));
    painter.drawText(aCaptionBuffer->rect(), AlignCenter, caption );
    painter.end();


    // inactive
    iCaptionBuffer->resize(captionWidth+4, s_titleHeight );
    painter.begin(iCaptionBuffer);
    painter.drawTiledPixmap(iCaptionBuffer->rect(), *iTitleBarTile);
    if(KindustrialHandler::titleShadow())
    {
        painter.drawImage(1, 1, shadow);
    }
    painter.setFont(s_titleFont);
    painter.setPen(options->color(Options::Font, false));
    painter.drawText(iCaptionBuffer->rect(), AlignCenter, caption );
    painter.end();

    captionBufferDirty = false;
}
